@@ -93,6 +93,8 @@ const MainWindow = new Lang.Class({
                             activate: this._setSelectionMode,
                             parameter_type: new GLib.VariantType('b'),
                             state: new GLib.Variant('b', false) },
+                          { name: 'go-world',
+                            activate: this._goWorld },
                           { name: 'select-all',
                             activate: this._selectAll },
                           { name: 'select-none',
@@ -108,12 +110,14 @@ const MainWindow = new Lang.Class({
         let grid = builder.get_object('main-panel');
         this._header = builder.get_object('header-bar');
         this.set_titlebar(this._header);
+        let [title, subtitle] = this._getTitle();
+        this._header.title = title;
+        this._header.subtitle = subtitle;
 
         let newButton = builder.get_object('new-button');
         this._pageWidgets[Page.WORLD].push(newButton);
 
         let goWorldButton = builder.get_object('world-button');
-        goWorldButton.connect('clicked', Lang.bind(this, this._goWorld));
         this._pageWidgets[Page.CITY].push(goWorldButton);
 
         let select = builder.get_object('select-button');
@@ -205,7 +209,7 @@ const MainWindow = new Lang.Class({
 
     _getTitle: function() {
         if (this._currentPage == Page.WORLD)
-            return ['', null];
+            return [_("World Weather"), null];
 
         let location = this._cityView.info.location;
         let city = location;
@@ -301,7 +305,7 @@ const MainWindow = new Lang.Class({
               license_type: Gtk.License.GPL_2_0,
               logo_icon_name: 'org.gnome.Weather.Application',
               version: pkg.version,
-              website: 'https://live.gnome.org/Design/Apps/Weather',
+              website: 'https://wiki.gnome.org/Apps/Weather',
               wrap_license: true,
               modal: true,
               transient_for: this
